% Leemos las imagenes y las coordenadas de las mascaras desde fichero
Fichero = fopen('train.txt');
Formato = '%s %f %f %f %f %*[^\n]';
A = textscan(Fichero,Formato,'Delimiter',',','EmptyValue',NaN);
fclose(Fichero);

imageFilename = string(A{1,1});
min_x = A{1,2};
min_y = A{1,3};
max_x = A{1,4};
max_y = A{1,5};

Tabla = table(imageFilename,min_x,min_y,max_x,max_y);
Tabla = rmmissing(Tabla);

% Tamaño de entrada de imagenes 720x576
Gasas = {size(Tabla)};
for i=1:size(Tabla)
    % Ajustar las dimensiones de las bboxes si alguna excede el tamaño de
    % imagen
    imageFilename2(i) = Tabla{i,1};
    min_x(i) = Tabla{i,2};
    min_y(i) = Tabla{i,3};
    max_x(i) = Tabla{i,4};
    max_y(i) = Tabla{i,5};
    
    if min_x(i) <= 0
        min_x(i) = 1;
    end
    if min_y(i) <= 0
        min_y(i) = 1;
    end
    if max_x(i) >= 720
        max_x(i) = 720;
    end
    if max_y(i) >= 576
        max_y(i) = 576;
    end
    Gasas{i,1} = [min_x(i), min_y(i),max_x(i)-min_x(i), max_y(i)-min_y(i)];
    
end

imageFilename2 = imageFilename2.';
BoundingBoxes_Gasas = table(imageFilename2,Gasas);
BoundingBoxes_Gasas = standardizeMissing(BoundingBoxes_Gasas,NaN);
BoundingBoxes_Gasas = rmmissing(BoundingBoxes_Gasas);

% Cargamos la red alexnet preentrenada
net = alexnet;

rng(0);
shuffledIndices = randperm(height(BoundingBoxes_Gasas));
idx = floor(0.9 * length(shuffledIndices));
trainingDataTbl = BoundingBoxes_Gasas(shuffledIndices(1:idx),:);
testDataTbl = BoundingBoxes_Gasas(shuffledIndices(idx+1:end),:);

% Obtenemos los almacenes de entrenamiento y prueba
imdsTrain = imageDatastore(trainingDataTbl{:,'imageFilename2'});
bldsTrain = boxLabelDatastore(trainingDataTbl(:,2:end));
imdstest = imageDatastore(testDataTbl{:,'imageFilename2'});
bldstest = boxLabelDatastore(testDataTbl(:,2:end));
trainingData = combine(imdsTrain,bldsTrain);
testData = combine(imdstest,bldstest);

Tamano_imagen = [224 224 3];
Num_Clases = 1; 

% Aumentamos el almacen de datos de entrenamiento y lo escalamos a [224 224]
TrainingData_aumentado = transform(trainingData,@augmentData);
trainingData_aumentado_escalado = transform(TrainingData_aumentado,@(data)PreprocessDatastore(data,Tamano_imagen));


% Creamos las anchor boxes para la red Yolo
trainingData_escalado = transform(trainingData,@(data)PreprocessDatastore(data,Tamano_imagen));
anchorBoxes = estimateAnchorBoxes(trainingData_escalado,5);

% Fijamos las opciones y las capas de entrenamiento
Opciones = trainingOptions('sgdm','InitialLearnRate',0.0001,'MaxEpochs',15,'MiniBatchSize',10,'Shuffle','every-epoch');
Capas = yolov2Layers(Tamano_imagen,Num_Clases,anchorBoxes,net,'relu5');

%Entrenamos la red YOLO
[Detector_Objetos, informacion] = trainYOLOv2ObjectDetector(trainingData_aumentado_escalado, Capas, Opciones);

% figure
plot(informacion.TrainingLoss);
grid on
xlabel('Number of Iterations')
ylabel('Training Loss for Each Iteration')


preprocessedTestData = transform(testData,@(data)PreprocessDatastore(data,Tamano_imagen));
detectionResults = detect(Detector_Objetos, preprocessedTestData);
[ap,recall,precision] = evaluateDetectionPrecision(detectionResults, preprocessedTestData);

figure
plot(recall,precision)
xlabel('Recall')
ylabel('Precision')
grid on
title(sprintf('Average Precision = %.2f',ap))


save('Detector_Alexnet_Yolov2.mat','Detector_Objetos');