% Reading bbox coordinates and image location from Train.txt
File = fopen('Train.txt');
Format = '%s %f %f %f %f %*[^\n]';
A = textscan(File,Format,'Delimiter',',','EmptyValue',NaN);
fclose(File);

imageFilename = string(A{1,1});
min_x = A{1,2};
min_y = A{1,3};
max_x = A{1,4};
max_y = A{1,5};

Tabla = table(imageFilename,min_x,min_y,max_x,max_y);
Tabla = rmmissing(Tabla);


% Image input size 720x5768
Gauzes = {size(Tabla)};
for i=1:size(Tabla)
    % Adjusting bbox dimension if any exceed the image input size
    imageFilename2(i) = Tabla{i,1};
    min_x(i) = Tabla{i,2};
    min_y(i) = Tabla{i,3};
    max_x(i) = Tabla{i,4};
    max_y(i) = Tabla{i,5};
    
    
    if min_x(i) <= 0
        min_x(i) = 1;
    end
    if min_y(i) <= 0
        min_y(i) = 1;
    end
    if max_x(i) >= 720
        max_x(i) = 720;
    end
    if max_y(i) >= 576
        max_y(i) = 576;
    end
    Gauzes{i,1} = [min_x(i), min_y(i), max_x(i)-min_x(i), max_y(i)-min_y(i)];
    
end

imageFilename2 = imageFilename2.';
BoundingBoxes_Gauzes = table(imageFilename2,Gauzes);
BoundingBoxes_Gauzes = standardizeMissing(BoundingBoxes_Gauzes,NaN);
BoundingBoxes_Gauzes = rmmissing(BoundingBoxes_Gauzes);

% Loading googlenet
net = googlenet;

% Train parameters
Input_size = [224 224 3];
Num_Clases = 1; 
numEpochs = 15;
miniBatchSize = 10;
learningRate = 0.001;
warmupPeriod = 1000;
l2Regularization = 0.0005;
penaltyThreshold = 0.5;
velocity = [];


rng(0);
shuffledIndices = randperm(height(BoundingBoxes_Gauzes));
idx = floor(0.9 * length(shuffledIndices));
trainingDataTbl = BoundingBoxes_Gauzes(shuffledIndices(1:idx),:);
testDataTbl = BoundingBoxes_Gauzes(shuffledIndices(idx+1:end),:);


% Train and test datastores
imdsTrain = imageDatastore(trainingDataTbl{:,'imageFilename2'});
bldsTrain = boxLabelDatastore(trainingDataTbl(:,2:end));
imdstest = imageDatastore(testDataTbl{:,'imageFilename2'});
bldstest = boxLabelDatastore(testDataTbl(:,2:end));
trainingData = combine(imdsTrain,bldsTrain);
testData = combine(imdstest,bldstest);

validateInputData(trainingData);
validateInputData(testData);

% Augmenting and preprocessing trainingdata
Augmented_TrainingData = transform(trainingData,@augmentData);
Preprocess_TrainingData = transform(Augmented_TrainingData,@(data)PreprocessDatastore(data,Input_size));

% Creating anchor boxes for YOLO
rng(0);
trainingData_Estimation = transform(trainingData,@(data)PreprocessDatastore(data,Input_size));
[anchorBoxes, meanIoU] = estimateAnchorBoxes(trainingData_Estimation,6);


area = anchorBoxes(:, 1).*anchorBoxes(:, 2);
[~, idx] = sort(area, 'descend');
anchorBoxes = anchorBoxes(idx, :);
anchorBoxMasks = {[1,2,3],[4,5,6]};

classNames = {'Gauzes'};
numClasses = size(classNames, 2);
numPredictorsPerAnchor = 5 + numClasses;

% Adding detection heads
lgraph = GooglenetFeatureExtractor(net, Tamano_imagen);

lgraph = addFirstDetectionHead(lgraph, anchorBoxMasks{1}, numPredictorsPerAnchor);
lgraph = addSecondDetectionHead(lgraph, anchorBoxMasks{2}, numPredictorsPerAnchor);

lgraph = connectLayers(lgraph, 'inception_5b-output', 'conv1Detection1');
lgraph = connectLayers(lgraph, 'relu1Detection1', 'upsample1Detection2');
lgraph = connectLayers(lgraph, 'inception_4c-output', 'depthConcat1Detection2/in2');
networkOutputs = ["conv2Detection1","conv2Detection2"];


mbqTrain = minibatchqueue(Preprocess_TrainingData, 2,"MiniBatchSize", miniBatchSize,"MiniBatchFcn", @(images, boxes, labels) createBatchData(images, boxes, labels, classNames), "MiniBatchFormat", ["SSCB", ""],"DispatchInBackground", true,"OutputCast", ["", "double"]);

% Convert layer graph to dlnetwork.
net = dlnetwork(lgraph);

% Create subplots for the learning rate and mini-batch loss.
fig = figure;
[lossPlotter, learningRatePlotter] = configureTrainingProgressPlotter(fig);

iteration = 0;
% Custom training loop.
for epoch = 1:numEpochs

    reset(mbqTrain);
    shuffle(mbqTrain);

    while(hasdata(mbqTrain))
        iteration = iteration + 1;

        [XTrain, YTrain] = next(mbqTrain);

        % Evaluate the model gradients and loss using dlfeval and the
        % modelGradients function.
        [gradients, state, lossInfo] = dlfeval(@modelGradients, net, XTrain, YTrain, anchorBoxes, anchorBoxMasks, penaltyThreshold, networkOutputs);

        % Apply L2 regularization.
        gradients = dlupdate(@(g,w) g + l2Regularization*w, gradients, net.Learnables);

        % Determine the current learning rate value.
        currentLR = piecewiseLearningRateWithWarmup(iteration, epoch, learningRate, warmupPeriod, numEpochs);

        % Update the network learnable parameters using the SGDM optimizer.
        [net, velocity] = sgdmupdate(net, gradients, velocity, currentLR);

        % Update the state parameters of dlnetwork.
        net.State = state;

        % Display progress.
        displayLossInfo(epoch, iteration, currentLR, lossInfo);  

        % Update training plot with new points.
        updatePlots(lossPlotter, learningRatePlotter, iteration, currentLR, lossInfo.totalLoss);
    end        
end

    
% Evaluamos el modelo
confidenceThreshold = 0.5;
overlapThreshold = 0.5;

% Create the test datastore.
preprocessedTestData = transform(testData, @(data)PreprocessDatastore(data,Input_size));

% Create a table to hold the bounding boxes, scores, and labels returned by
% the detector. 
numImages = size(testDataTbl, 1);
results = table('Size', [0 3], 'VariableTypes', {'cell','cell','cell'},'VariableNames', {'Boxes','Scores','Labels'});

mbqTest = minibatchqueue(preprocessedTestData, 1,"MiniBatchSize", miniBatchSize, "MiniBatchFormat", "SSCB");

% Run detector on images in the test set and collect results.
while hasdata(mbqTest)
    % Read the datastore and get the image.
    XTest = next(mbqTest);
    
    % Run the detector.
    [bboxes, scores, labels] = yolov3Detect(net, XTest, networkOutputs, anchorBoxes, anchorBoxMasks, confidenceThreshold, overlapThreshold, classNames);
    
    % Collect the results.
    tbl = table(bboxes, scores, labels, 'VariableNames', {'Boxes','Scores','Labels'});
    results = [results; tbl];
end

% Evaluate the object detector using Average Precision metric.
[ap, recall, precision] = evaluateDetectionPrecision(results, preprocessedTestData);


% Plot precision-recall curve.
figure
plot(recall, precision)
xlabel('Recall')
ylabel('Precision')
grid on
title(sprintf('Average Precision = %.2f', ap))

save('Detector_Googlenet.mat','net');
save('Salidas_Red_Googlenet','networkOutputs')
save('AnchorBox_Googlenet','anchorBoxes');
save('AnchorBoxMask_Googlenet','anchorBoxMasks');