% Cargamos la tabla con las imagenes y bbox
data = load('BoundingBoxes_Herramientas.mat');
imageFilename = data.gTruth.DataSource.Source;
Pinza = data.gTruth.LabelData{:,1};
BoundingBoxes_Herramientas = table(imageFilename,Pinza);

% Cargamos la red ResNet-50 preentrenada
net = resnet50;

rng(0);
shuffledIndices = randperm(height(BoundingBoxes_Herramientas));
idx = floor(0.9 * length(shuffledIndices));
trainingDataTbl = BoundingBoxes_Herramientas(shuffledIndices(1:idx),:);
testDataTbl = BoundingBoxes_Herramientas(shuffledIndices(idx+1:end),:);

% Obtenemos los almacenes de entrenamiento y prueba
imdsTrain = imageDatastore(trainingDataTbl{:,'imageFilename'});
bldsTrain = boxLabelDatastore(trainingDataTbl(:,2:end));
imdstest = imageDatastore(testDataTbl{:,'imageFilename'});
bldstest = boxLabelDatastore(testDataTbl(:,2:end));
trainingData = combine(imdsTrain,bldsTrain);
testData = combine(imdstest,bldstest);

Tamano_imagen = [224 224 3];
Num_Clases = 1; 

% Aumentamos el almacen de datos de entrenamiento y lo escalamos a [224 224]
TrainingData_aumentado = transform(trainingData,@augmentData);
trainingData_aumentado_escalado = transform(TrainingData_aumentado,@(data)PreprocessDatastore(data,Tamano_imagen));


% Creamos las anchor boxes para la red Yolo
trainingData_escalado = transform(trainingData,@(data)PreprocessDatastore(data,Tamano_imagen));
anchorBoxes = estimateAnchorBoxes(trainingData_escalado,5);

% Fijamos las opciones y las capas de entrenamiento
Opciones = trainingOptions('sgdm','InitialLearnRate',0.0001,'MaxEpochs',15,'MiniBatchSize',10,'Shuffle','every-epoch');
% Capas = yolov2Layers(Tamano_imagen,Num_Clases,anchorBoxes,net,'activation_49_relu','ReorgLayerSource','activation_47_relu');
Capas = yolov2Layers(Tamano_imagen,Num_Clases,anchorBoxes,net,'activation_40_relu');

%Entrenamos la red YOLO
[Detector_Objetos, informacion] = trainYOLOv2ObjectDetector(trainingData_aumentado_escalado, Capas, Opciones);
figure
plot(informacion.TrainingLoss);
grid on
xlabel('Number of Iterations')
ylabel('Training Loss for Each Iteration')

preprocessedTestData = transform(testData,@(data)PreprocessDatastore(data,Tamano_imagen));
detectionResults = detect(Detector_Objetos, preprocessedTestData);
[ap,recall,precision] = evaluateDetectionPrecision(detectionResults, preprocessedTestData);

figure
plot(recall,precision)
xlabel('Recall')
ylabel('Precision')
grid on
title(sprintf('Average Precision = %.2f',ap))

save('Detector_Resnet_Yolov2.mat','Detector_Objetos');