% Cargamos la tabla con las imagenes y bbox
data = load('BoundingBoxes_Herramientas.mat');
imageFilename = data.gTruth.DataSource.Source;
Pinza = data.gTruth.LabelData{:,1};
BoundingBoxes_Herramientas = table(imageFilename,Pinza);

%Cargamos la red DarkNet-53 preentrenada
net = darknet53;

% Train parameters
Input_size = [256 256 3];
Num_Clases = 1; 
numEpochs = 40;
miniBatchSize = 10;
learningRate = 0.0005;
warmupPeriod = 1500;
l2Regularization = 0.0005;
penaltyThreshold = 0.5;
velocity = [];

rng(0);
shuffledIndices = randperm(height(BoundingBoxes_Herramientas));
idx = floor(0.9 * length(shuffledIndices));
trainingDataTbl = BoundingBoxes_Herramientas(shuffledIndices(1:idx),:);
testDataTbl = BoundingBoxes_Herramientas(shuffledIndices(idx+1:end),:);

% Obtenemos los almacenes de entrenamiento y prueba
imdsTrain = imageDatastore(trainingDataTbl{:,'imageFilename'});
bldsTrain = boxLabelDatastore(trainingDataTbl(:,2:end));
imdstest = imageDatastore(testDataTbl{:,'imageFilename'});
bldstest = boxLabelDatastore(testDataTbl(:,2:end));
trainingData = combine(imdsTrain,bldsTrain);
testData = combine(imdstest,bldstest);

% Augmenting and preprocessing trainingdata
Augmented_TrainingData = transform(trainingData,@augmentData);
Preprocess_TrainingData = transform(Augmented_TrainingData,@(data)PreprocessDatastore(data,Input_size));

% Creating anchor boxes for YOLO
rng(0);
trainingData_Estimation = transform(trainingData,@(data)PreprocessDatastore(data,Input_size));
[anchorBoxes, meanIoU] = estimateAnchorBoxes(trainingData_Estimation,6);


area = anchorBoxes(:, 1).*anchorBoxes(:, 2);
[~, idx] = sort(area, 'descend');
anchorBoxes = anchorBoxes(idx, :);
anchorBoxMasks = {anchorBoxes(1:3,:) 
    anchorBoxes(4:6,:)
    };

classNames = {'Pinza'};
numClasses = size(classNames, 2);

% Eliminamos las capas de clasificación y añadimos las propias de YOLOv3
lgraph = removeLayers(net, {'avg1' 'conv53' 'softmax_' 'output'});
net = dlnetwork(lgraph);
net = yolov3ObjectDetector(net, classNames,anchorBoxMasks,'DetectionNetworkSource',{'res23','res12'});

mbqTrain = minibatchqueue(Preprocess_TrainingData, 2,"MiniBatchSize", miniBatchSize,"MiniBatchFcn", @(images, boxes, labels) createBatchData(images, boxes, labels, classNames), "MiniBatchFormat", ["SSCB", ""],"DispatchInBackground", true,"OutputCast", ["", "double"]);

% Create subplots for the learning rate and mini-batch loss.
fig = figure;
[lossPlotter, learningRatePlotter] = configureTrainingProgressPlotter(fig);

iteration = 0;
% Custom training loop.
for epoch = 1:numEpochs

    reset(mbqTrain);
    shuffle(mbqTrain);

    while(hasdata(mbqTrain))
        iteration = iteration + 1;

        [XTrain, YTrain] = next(mbqTrain);

        % Evaluate the model gradients and loss using dlfeval and the
        % modelGradients function.
        [gradients, state, lossInfo] = dlfeval(@modelGradients, net, XTrain, YTrain, penaltyThreshold);

        % Apply L2 regularization.
        gradients = dlupdate(@(g,w) g + l2Regularization*w, gradients, net.Learnables);

        % Determine the current learning rate value.
        currentLR = piecewiseLearningRateWithWarmup(iteration, epoch, learningRate, warmupPeriod, numEpochs);

        % Update the network learnable parameters using the SGDM optimizer.
        [net.Learnables, velocity] = sgdmupdate(net.Learnables, gradients, velocity, currentLR);

        % Update the state parameters of dlnetwork.
        net.State = state;

        % Display progress.
        displayLossInfo(epoch, iteration, currentLR, lossInfo);  

        % Update training plot with new points.
        updatePlots(lossPlotter, learningRatePlotter, iteration, currentLR, lossInfo.totalLoss);
    end        
end


% Create the test datastore.
preprocessedTestData = transform(testData, @(data)PreprocessDatastore(data,Input_size));

results = detect(net,preprocessedTestData,'MiniBatchSize', miniBatchSize);

% Evaluate the object detector using Average Precision metric.
[ap, recall, precision] = evaluateDetectionPrecision(results, preprocessedTestData);


% Plot precision-recall curve.
figure
plot(recall, precision)
xlabel('Recall')
ylabel('Precision')
grid on
title(sprintf('Average Precision = %.2f', ap))

save('Detector_Darknet53.mat','net');