% Load Resnet YOlOv3
Yolov3 = load('Detector_Resnet50.mat','net');
net = Yolov3.net;

% Load rest of detection parameters 
confidenceThreshold = 0.5;
overlapThreshold = 0.5;
classNames = {'Gauze'};
Outputs = load('Salidas_Red_Resnet50.mat','networkOutputs');
networkOutputs = Outputs.networkOutputs;
Bboxes = load('AnchorBox_Resnet50.mat','anchorBoxes');
anchorBoxes = Bboxes.anchorBoxes;
Bboxes_mask = load('AnchorBoxMask_Resnet50.mat','anchorBoxMasks');
anchorBoxMasks = Bboxes_mask.anchorBoxMasks;

% Choose video for detection
N = 2;
% Loading video chosen
videoObject = VideoReader(strcat('VID000',num2str(N),'.mp4'));

% Creating output video
OutputVideo = VideoWriter(strcat('Resnet50_YoloV3_DetectedVideo_000',num2str(N)),'MPEG-4'); OutputVideo.FrameRate = videoObject.FrameRate; open(OutputVideo);

fps = 0;
avgfps = [];

% Extract the frame from the movie structure.
for i=1:videoObject.NumFrames
    % Reading frame% Leemos el frame actual
    thisFrame = read(videoObject, i);
    thisFrame = dlarray(im2single(imresize(thisFrame,[224 224])),'SSCB');
    
    executionEnvironment = "auto";
    if (executionEnvironment == "auto" && canUseGPU) || executionEnvironment == "gpu"
        thisFrame = gpuArray(thisFrame);
    end

    tic;
    % Detecting objects on frame
    [bbox, score, label] = yolov3Detect(net,thisFrame,networkOutputs, anchorBoxes, anchorBoxMasks, confidenceThreshold, overlapThreshold, classNames);
    
    newt = toc;
    fps = .9*fps + .1*(1/newt);
    avgfps = [avgfps; fps];
    
    clear yolov3Detect 
    thisFrame = extractdata(thisFrame);
    thisFrame = gather(thisFrame);
    
     % if object found, it inserts the bbox and label
    if (~isempty(score{1}))
        thisFrame = insertObjectAnnotation(thisFrame,'rectangle',bbox{1},label{1}); 
    end

    % Saving frame on outputVideo
    writeVideo(OutputVideo,thisFrame);

end
Framerate = CalculoMediaFps(avgfps);
Texto = ['Fps = ',num2str(Framerate)];
disp(Texto);

close(OutputVideo);